using System;
using System.Data;

namespace HasErrorExample
{
	class Class1
	{
		private static void FindDuplicate(DataSet source, DataSet dest)
		{
			DataRow[] r;

			for (int i=0;i<dest.Tables[0].Rows.Count;i++)
			{
				r = source.Tables[0].Select("au_id='" + dest.Tables[0].Rows[i]["au_id"] + "'");
				if (r.Length > 0)
				{
					dest.Tables[0].Rows[i].RowError = "Error, duplicate primary key value"; 
				}
			}
		}

		private static void PrintErrors(DataSet d)
		{
			for (int i=0;i<d.Tables[0].Rows.Count;i++)
			{
				if (d.Tables[0].Rows[i].HasErrors)
				{
					Console.WriteLine(d.Tables[0].Rows[i].RowError.ToString()); 
				}
			}
		}

		static void Main(string[] args)
		{
			// Read XML Schema containing table structure.
			DataSet ds = new DataSet("DS");
			ds.ReadXmlSchema("F:\\ADO.NET\\Chapter5\\AuthorsSchema.XML");

			// Add a unique constraint on the au_id column
			ds.Tables[0].Constraints.Add("ConstraintID",ds.Tables[0].Columns["au_id"],true);

			// Add the first record
			DataRow dr = ds.Tables[0].NewRow();
			dr["au_id"] = "222-22-2222";
			dr["au_fname"] = "Fabio Claudio";
			dr["au_lname"] = "Ferracchiati";
			dr["contract"] = true;
			ds.Tables[0].Rows.Add(dr);
			ds.Tables[0].AcceptChanges();

			// Another DataSet filled with XML data
			DataSet dsXML = new DataSet("XMLDataSet");
			dsXML.ReadXmlSchema("F:\\ADO.NET\\Chapter5\\AuthorsSchema.XML");
			dsXML.ReadXml("F:\\ADO.NET\\Chapter5\\Authors.XML");

			// Looks for duplicated records
			FindDuplicate(ds, dsXML);

			// If found errors, print out them
			// Otherwise merge the dataset
			if(!dsXML.HasErrors)
			{
				ds.Merge(dsXML);
				Console.WriteLine("Merge done!");
			}
			else
			{
				PrintErrors(dsXML);
			}
		}
	}
}
