using System;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace DayOfDotNet
{
	public class DynamicEmbeddedDataGrid : System.Web.UI.Page
	{
		protected DataGrid BookDataGrid;
		private DataSet BookDataSet;

		private void Page_Load(object sender, System.EventArgs e)
		{
			//Set a default ISBN
			string isbn;
			
			//When the page is loaded, create the BookDataSet.
			//Create a BooksData class instance
			BooksData _booksData = new BooksData();
			
			//If there is an ISBN in the QueryString, use it instead.
			//and invoke GetBookInfo() passing in the ISBN string.
			if(HttpContext.Current.Request.QueryString["isbn"] != null)
			{
				isbn = HttpContext.Current.Request.QueryString["isbn"];
				BookDataSet = _booksData.GetBookInfo(isbn);
			}
			else
			{
				BookDataSet = _booksData.GetAllBookInfoDataSet();
				
			}

			BookDataGrid.DataSource = BookDataSet.Tables["Books"];
			BookDataGrid.DataBind();
		}

		//Use the OnItemCreated event handler to dynamically add an embedded DataGrid
		protected void BookDataGrid_OnItemDataBound(object sender, DataGridItemEventArgs e)
		{
			//When each row is created in the DataGrid, eval the ItemType
			if(e.Item.ItemType == ListItemType.Item || 
				e.Item.ItemType == ListItemType.AlternatingItem)
			{
				//If the ItemType is Item or AlternatingItem, 
				//Create a new DataGrid object named AuthorsDataGrid
				DataGrid AuthorsDataGrid = new DataGrid();
				
				//Format the DataGrid to look cool.
				AuthorsDataGrid.ItemStyle.Font.Name = "Verdana";
				AuthorsDataGrid.ItemStyle.Font.Size = FontUnit.XSmall;
				AuthorsDataGrid.BorderWidth = (Unit)1;
				AuthorsDataGrid.ShowHeader = false;
				AuthorsDataGrid.AlternatingItemStyle.BackColor = 
					Color.FromName("Tan");
				
				//Do not autogenerate columns.
				AuthorsDataGrid.AutoGenerateColumns = false;
				
				//For the only column in the AuthorsDataGrid, create a HyperLinkColumn.
				HyperLinkColumn hlc = new HyperLinkColumn();
				
				//Format the link to send a mail message.
				hlc.DataNavigateUrlField = "Email";
				hlc.DataNavigateUrlFormatString = "mailto:{0}";
				hlc.DataTextField = "AuthorName";
				hlc.ItemStyle.Wrap = false;
				
				//Add the HyperlinkColumn to the AuthorsDataGrid.
				AuthorsDataGrid.Columns.Add(hlc);
				
				//Get the Authors DataView and filter it for the current ISBN
				DataView _authors = BookDataSet.Tables["Authors"].DefaultView;
				_authors.RowFilter = "ISBN='" + e.Item.Cells[0].Text + "'";
				
				//Bind the DataGrid.
				AuthorsDataGrid.DataSource = _authors;
				AuthorsDataGrid.DataBind();
				
				//Add the AuthorsDataGrid to the BooksDataGrid.
				e.Item.Cells[3].Controls.Add(AuthorsDataGrid);
			}
		}
	}
}