using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Collections;

namespace CustomControls.DropDownList
{
	[DefaultProperty("DataSource"), 
	DefaultEvent("SelectedIndexChanged"),
	ToolboxData("<{0}:SimpleListControl runat=server></{0}:SimpleListControl>")]
	public abstract class SimpleListControl : WebControl
	{
		private static readonly object EventSelectedIndexChanged = new object();

		#region Ӽ 
		private string dataTextField;
		private string dataValueField;
		private int selectedIndex = -1;
		private ListItem selectedItem;
		private ListItemCollection items = new ListItemCollection();
		private IEnumerable dataSource;
		#endregion

		#region Ӽ
		[Bindable(true),
		Category("Data"),
		DefaultValue(null),
		Description("Ʈѿ ̵ϱ ϴ  ҽ."),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public virtual IEnumerable DataSource {
			get {return dataSource;}
			set {dataSource = value;}
		}

		[Bindable(true), 
		Category("Data"), 
		DefaultValue(null)]
		public virtual ListItemCollection Items {
			get {return items;}
		}

		[Bindable(true), 
		Category("Data"), 
		DefaultValue("")]
		public virtual string DataTextField {
			get {return dataTextField;}
			set {dataTextField = value;}
		}

		[Bindable(true), 
		Category("Data"), 
		DefaultValue("")]
		public virtual string DataValueField {
			get {return dataValueField;}
			set {dataValueField = value;}
		}

		[Browsable(false)]
		public virtual int SelectedIndex {
			get {return selectedIndex;}
			set {
				if (value < -1 || (value > items.Count - 1))
					throw new ArgumentException("  ׸  Ѿϴ.");

				//   
				foreach (ListItem item in items )
					if (item.Selected)	{
						item.Selected = false;
						break;
					}

				if (value == -1) {
					selectedIndex = -1;
					selectedItem = null;
				}
				else {
					selectedItem = items[value];
					selectedItem.Selected = true;
					selectedIndex = value;
				}
			}
		}
		
		[Browsable(false)]
		public virtual ListItem SelectedItem {
			get {return selectedItem;}
		}
		#endregion

		#region ̺Ʈ
		protected virtual void OnSelectedIndexChanged(EventArgs e) 
		{
			EventHandler handler = (EventHandler)Events[EventSelectedIndexChanged];
			if (handler != null) handler(this, e);
		}

		[Category("Action"),
		Description("SelectedIndex Ӽ  Ǹ ߻.")]
		public event EventHandler SelectedIndexChanged {
			add {Events.AddHandler(EventSelectedIndexChanged, value);}
			remove {Events.RemoveHandler(EventSelectedIndexChanged, value);}
		}
		#endregion

		#region ޼ҵ

		public SimpleListControl() : base(HtmlTextWriterTag.Select)	{}

		public override void DataBind() 
		{
			// ׸  
			items.Clear();

			bool isOneField;
			string text = String.Empty;
			string val = String.Empty;

			isOneField = dataTextField == null || dataTextField == String.Empty;

			if (!isOneField) {
				text = dataTextField;
				if (dataValueField == null || dataValueField == String.Empty)
					val = dataTextField;
				else
					val = dataValueField;
			}
		
			foreach (object dsItem in dataSource) {
				ListItem item;
				if (isOneField)	{
					item = new ListItem(dsItem.ToString());
				}
				else {
					item = new ListItem();
					item.Text = DataBinder.GetPropertyValue(dsItem, text).ToString();
					item.Value = DataBinder.GetPropertyValue(dsItem, val).ToString();
				}
				items.Add(item);
			}

			TrackViewState();
		}

		protected override void LoadViewState(object savedState)
		{
			if (savedState != null)	{
				object[] stateArr = (object[])savedState;

				if (stateArr[0] != null)
					base.LoadViewState(stateArr[0]);
				if (stateArr[0] != null)
					dataTextField = stateArr[1].ToString();
				if (stateArr[1] != null)
					dataValueField = stateArr[2].ToString();
				selectedIndex = (int)stateArr[3];
				((IStateManager)items).LoadViewState(stateArr[4]);
				/****************************************************
				 * ((IStateManager)items).LoadViewState(stateArr[4])
				 *   Ʒ  ڵ ʿϴ.
				 ****************************************************/
				/*
				//items 
				string[] itemsText = (string[])stateArr[4];
				string[] itemsValue = (string[])stateArr[5];

				items.Clear();
				for (int i=0; i<itemsText.Length; i++) {
					items.Add(new ListItem(itemsText[i], itemsValue[i]));
				}
				*/

				if (selectedIndex != -1) {
					items[selectedIndex].Selected = true;
					selectedItem = items[selectedIndex];
				}
			}
		}

		protected override object SaveViewState()
		{
			object[] stateArr = new object[5];
			stateArr[0] = base.SaveViewState();
			stateArr[1] = dataTextField;
			stateArr[2] = dataValueField;
			stateArr[3] = selectedIndex;
			stateArr[4] = ((IStateManager)items).SaveViewState();

			/**********************************************
			 * ((IStateManager)items).SaveViewState()
			 *   Ʒ  ڵ ʿϴ.
			 **********************************************/
			/*
			//ListItemCollection   ʵǹǷ
			//迭  Ѵ.
			string[] itemsText = new string[items.Count];
			string[] itemsValue = new string[items.Count];
			for (int i=0; i<items.Count; i++) {
				itemsText[i] = items[i].Text;
				itemsValue[i] = items[i].Value;
			}

			stateArr[4] = itemsText;
			stateArr[5] = itemsValue;
			*/
			
			return (object)stateArr;
		}

		protected override void TrackViewState() 
		{
			base.TrackViewState();

			((IStateManager)items).TrackViewState();
		}
		#endregion
	}
}