using System;
using System.Web;
using System.IO;
using System.Collections;
using System.Configuration;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Text;
using System.Drawing.Drawing2D;

namespace MyWebLibs.Altercast
{
	public enum ImageSize : byte {All, Large, Middle, Small}

	/// <summary>
	/// Altercast   Դϴ.
	/// </summary>
	public class Altercast
	{
		#region   
		private string _baseDir;
		private string _imgHead;
		private string _altercastDirName;
		private ImageFormat _imgFormatL;
		private ImageFormat _imgFormatM;
		private ImageFormat _imgFormatS;
		private ArrayList _kinds;
		//ϴ ̹  ũ⸦  Ѵ.
		private int _orgWidth = -1;

		private HttpContext _context;

		private string _uniqueID;
		#endregion

		#region  Ӽ
		public string BaseDir {
			get {return _baseDir;}
			set {_baseDir = value;}
		}

		public string ImageHead {
			get {return _imgHead;}
			set {_imgHead = value;}
		}

		public string AltercastDirName {
			get {return _altercastDirName;}
			set {_altercastDirName = value;}
		}

		public string AltercastDirPath {
			get {
				string dirPath = _baseDir + _altercastDirName;
				return dirPath[dirPath.Length-1] == '/' ? dirPath : dirPath + "/";
			}
		}

		public ImageFormat ImageFormatL 
		{
			get {return _imgFormatL;}
			set {_imgFormatL = value;}
		}

		public ImageFormat ImageFormatM {
			get {return _imgFormatM;}
			set {_imgFormatM = value;}
		}

		public ImageFormat ImageFormatS {
			get {return _imgFormatS;}
			set {_imgFormatS = value;}
		}

		public int OriginalWidth {
			get {return _orgWidth;}
			set {_orgWidth = value;}
		}

		public ArrayList Kinds {
			get {return _kinds;}
		}
		#endregion

		#region  ޼ҵ 
		public Altercast(string baseDir, string imgHead, string altercastDirName,
			ImageFormat imgFmtL, ImageFormat imgFmtM, ImageFormat imgFmtS, 
			ArrayList kinds) 
		{
			Init(baseDir, imgHead, altercastDirName,
			imgFmtL, imgFmtM, imgFmtS, kinds);
		}

		public Altercast(string defName) 
		{
			Hashtable infos = (Hashtable)ConfigurationSettings.GetConfig("Altercast/AltercastDef");

			AltercastConfiguration.ConfigInfo cf = (AltercastConfiguration.ConfigInfo)infos[defName];

			Init(cf.BaseDir, cf.ImageNameHead, cf.AltercastDirName,
				GetImageFormat(cf.ImageFormatL), 
				GetImageFormat(cf.ImageFormatM), 
				GetImageFormat(cf.ImageFormatS), cf.Kinds);
		}

		public void SetUniqueID() {
			_uniqueID = DateTime.Now.Ticks.ToString();
		}

		public void SetUniqueID(string uniqueID) {
			_uniqueID = uniqueID;
		}

		public string GetUniqueID() {
			return _uniqueID;
		}

		public string GetImagePathByNo(int no, ImageSize imgSize, out string front, out string ext) {

			front = GetImageNameFront(no, _imgHead, imgSize.ToString()[0] + "");
			ext = GetImageExtention(GetImageFormatPerSize(imgSize));

			return  _baseDir + front + ext;
		}

		// Ǵ (isNew) ƴϸ SetUniqueID()  ȣ 
		//_uniqueID  ¿  ޼ҵ带 ȣѴ.
		public string GetAltImagePathByNo(int no, ImageSize imgSize, 
			string kindSimpler, bool isNew, GenerateParams gp) 
		{
			string retName;

			string front;
			string ext;
			
			GetImagePathByNo(no, imgSize, out front, out ext);
			
			string frontName = front;
			string backendName = kindSimpler + ext;

			if (isNew) {
				string normalName = frontName + ext;

				retName = frontName + GetUniqueID() + backendName;

				ClearPrevAltImages(AltercastDirPath, frontName, backendName);

				bool success = GenerateImage(normalName, 
					gp.OverImageName, 
					gp.AppendString, gp.StringDependentOnImage, 
					gp.StringRotate, gp.StringAlpha,
					gp.StringColor, gp.StringFont,
					gp.StringSize, gp.OverXY, gp.OverRotate, 
					gp.OverAlpha, gp.OverPercent, 
					gp.OverStringPosXY, gp.OverStringHAuto, 
					gp.OverStringVAuto, retName, 
					imgSize, kindSimpler);

				if (!success)
					retName = String.Empty;
			}
			else {
				//retName = front + "_" + imgSize.ToString()[0] + GetUniqueID() + kindSimpler + ext;
				retName = frontName + GetUniqueID() + kindSimpler + ext;
				
			}

			return AltercastDirPath + retName;
		}

		public string GetKindNameByKindSimpler(string kindSimpler) {
			foreach(Kind kind in _kinds) {
				if (kind.Simpler == kindSimpler)
					return kind.Nmae;
			}

			return null;
		}

		
		public string GetImageExtention(ImageFormat imgFmt) 
		{
			if (imgFmt == ImageFormat.Gif)
				return ".gif";
			else if (imgFmt == ImageFormat.Jpeg)
				return ".jpg";
			else if (imgFmt == ImageFormat.Png)
				return ".png";
			else
				return string.Empty;
		}

		public ImageFormat GetImageFormatPerSize(ImageSize imgSize) 
		{
			switch (imgSize) {
				case ImageSize.Large :
					return _imgFormatL;
				case ImageSize.Middle :
					return _imgFormatM;
				default :
					return _imgFormatS;
			}
		}

		// ɽƮ  .
		public void ClearPrevAltImages(string dir, string frontName, string backendName) 
		{
			string filePath = _context.Server.MapPath(dir);
			string [] files = Directory.GetFiles(filePath, frontName + "*" + backendName);
			foreach (string file in files)
				File.Delete(file);

		}

		public bool GenerateImage(string baseImage, string overImage, 
			string appendString, bool stringDependentOnImage, 
			int stringRotate,int stringAlpha, 
			string stringColor, string stringFont, 
			int stringSize, int[] overXY, int overRotate, 
			int overAlpha, int overPercent, 
			int[] overStringPosXY, bool overStringHAuto, 
			bool overStringVAuto, string resultImage, 
			ImageSize imgSize, string kindSimpler) {

			string kindName = GetKindNameByKindSimpler(kindSimpler);
			if (kindName == null)
				throw new Exception(kindSimpler + "  Ŭ  Դϴ.");

			ImageFormat imgFormat = GetImageFormatPerSize(imgSize);

			Bitmap oBitmap;

			bool success = GenerateImage(_baseDir + baseImage, 
				_baseDir + kindName + "/" + overImage, appendString, 
				stringDependentOnImage, stringRotate, stringAlpha, 
				stringColor, stringFont, stringSize, 
				overXY, overRotate, overAlpha, overPercent, 
				overStringPosXY, overStringHAuto, overStringVAuto, 
				/*AltercastDirPath + resultImage,*/ _orgWidth,
				_context, /*imgFormat,*/ out oBitmap);

			oBitmap.Save(_context.Server.MapPath(AltercastDirPath + resultImage), imgFormat);
			oBitmap.Dispose();

			return success;
		}

		public bool GenerateImageAndSendToClient(string baseImage, string overImage, 
			string appendString, bool stringDependentOnImage, 
			int stringRotate,int stringAlpha, 
			string stringColor, string stringFont, 
			int stringSize, int[] overXY, int overRotate, 
			int overAlpha, int overPercent, 
			int[] overStringPosXY, bool overStringHAuto, 
			bool overStringVAuto,  
			ImageSize imgSize, string kindSimpler) 
		{

			string kindName = GetKindNameByKindSimpler(kindSimpler);
			if (kindName == null)
				throw new Exception(kindSimpler + "  Ŭ  Դϴ.");

			ImageFormat imgFormat = GetImageFormatPerSize(imgSize);

			Bitmap oBitmap;

			bool success = GenerateImage(_baseDir + baseImage, 
				_baseDir + kindName + "/" + overImage, appendString, 
				stringDependentOnImage, stringRotate, stringAlpha, 
				stringColor, stringFont, stringSize, 
				overXY, overRotate, overAlpha, overPercent, 
				overStringPosXY, overStringHAuto, overStringVAuto, 
				_orgWidth,	_context, out oBitmap);

			oBitmap.Save(_context.Response.OutputStream, imgFormat);
			oBitmap.Dispose();

			return success;
		}
		#endregion

		#region   ޼ҵ 
		static public bool GenerateImage(string baseImagePath, 
			string overImagePath, string appendString, 
			bool stringDOI, int stringRotate, int stringAlpha, 
			string stringColor, string stringFont, 
			int stringSize, int[] overXY, int overRotate, 
			int overAlpha, int overPercent, 
			int[] overStringPosXY, bool overStringHAuto, 
			bool overStringVAuto, /*string resultImagePath,*/ int orgWidth,
			HttpContext context, /*ImageFormat imgFormat,*/ out Bitmap oResult)	{

			try 
			{
				Bitmap oBaseImage = new Bitmap(context.Server.MapPath(baseImagePath));
				Bitmap oOverImage = new Bitmap(context.Server.MapPath(overImagePath));

				//orgWidth   ʾ ⺻ ̹ ũ⸦ 
				if (orgWidth == -1)
					orgWidth = oBaseImage.Width;

				// ̹  Graphics ü 
				Graphics oGraphics = Graphics.FromImage( oBaseImage );

				float scaleValue1 = (float)oBaseImage.Width / (float)orgWidth;
				oGraphics.ScaleTransform(scaleValue1, scaleValue1);

				//  
				GraphicsState transState = oGraphics.Save();

				float scaleValue2 = (float)overPercent / 100;
				oGraphics.ScaleTransform(scaleValue2, scaleValue2, MatrixOrder.Append);

				// ũ ̹ ǥ
				int coordinateX = (int)(overXY[0] * scaleValue1);
				int coordinateY = (int)(overXY[1] * scaleValue1);

				oGraphics.TranslateTransform(coordinateX, coordinateY, MatrixOrder.Append);

				ColorMatrix myColorMatrix = new ColorMatrix();
				myColorMatrix.Matrix00 = 1.00f; // Red
				myColorMatrix.Matrix11 = 1.00f; // Green
				myColorMatrix.Matrix22 = 1.00f; // Blue
				myColorMatrix.Matrix33 = ((float)overAlpha / 100); // alpha
				myColorMatrix.Matrix44 = 1.00f; // w

				ImageAttributes imageAttr = new ImageAttributes();
				imageAttr.SetColorMatrix(myColorMatrix);

				Color lowerColor = Color.FromArgb(255,255,255);
				Color upperColor = Color.FromArgb(255,255,255);
				imageAttr.SetColorKey(lowerColor, upperColor);

				oGraphics.RotateTransform(overRotate);

				oGraphics.CompositingMode = CompositingMode.SourceOver;
				Rectangle rectImg = new Rectangle(0, 0, oOverImage.Width, oOverImage.Height);
				oGraphics.DrawImage(oOverImage,         // Image
					rectImg,            // Dest. rect.
					0,               // srcX
					0,               // srcY
					oOverImage.Width,             // srcWidth
					oOverImage.Height,             // srcHeight
					GraphicsUnit.Pixel, // srcUnit
					imageAttr);      // ImageAttributes

				FontFamily fontFamily = new FontFamily(stringFont);
				Font font = new Font(
					fontFamily,
					stringSize, FontStyle.Bold,
					GraphicsUnit.Pixel);

				int rectStrX = 0;
				int rectStrY = 0;
				if (stringDOI) {
					StringFormat stringFormat = new StringFormat();

					if (overStringPosXY[0] != -1 && !overStringHAuto)
						rectStrX = overStringPosXY[0];
					else {
						rectStrX = -(oOverImage.Width/2);
						stringFormat.Alignment = StringAlignment.Center;
					}

					if (overStringPosXY[1] != -1 && !overStringVAuto)
						rectStrY = overStringPosXY[1];
					else {
						rectStrY = -(oOverImage.Height/2);
						stringFormat.LineAlignment = StringAlignment.Center;
					}

					Rectangle rectStr = new Rectangle(rectStrX, rectStrY, oOverImage.Width*2, oOverImage.Height*2);
					SolidBrush solidBrush = new SolidBrush(Color.FromName(stringColor));

					//ڿ 
					oGraphics.DrawString(appendString, font, solidBrush, rectStr, stringFormat);
				}
				else {
					//ο ̹ ü  
					// İ ϱ 
					Bitmap oBTmp = new Bitmap(oBaseImage.Width, (int)font.GetHeight(oGraphics)+1, oGraphics);
					Graphics oGTmp = Graphics.FromImage(oBTmp);

					PointF drawPoint = new PointF(0f, 0f);
					SolidBrush solidBrush = new SolidBrush(Color.FromName(stringColor));
					oGTmp.DrawString(appendString, font, solidBrush, drawPoint);

					oGraphics.ResetTransform();
					//տ   
					oGraphics.Restore(transState);
					//oGraphics.ScaleTransform(scaleValue1, scaleValue1);

					if (overStringPosXY[0] != -1)
						rectStrX = (int)(scaleValue1 * overStringPosXY[0]);

					if (overStringPosXY[1] != -1)
						rectStrY = (int)(scaleValue1 * overStringPosXY[1]);
					oGraphics.TranslateTransform(rectStrX, rectStrY, MatrixOrder.Append);
					oGraphics.RotateTransform(stringRotate);

					myColorMatrix.Matrix33 = ((float)stringAlpha / 100); // alpha

					imageAttr.SetColorMatrix(myColorMatrix);

					rectImg = new Rectangle(0, 0, oBTmp.Width, oBTmp.Height);
					oGraphics.DrawImage(oBTmp,         // Image
						rectImg,            // Dest. rect.
						0,               // srcX
						0,               // srcY
						oBTmp.Width,             // srcWidth
						oBTmp.Height,             // srcHeight
						GraphicsUnit.Pixel, // srcUnit
						imageAttr);      // ImageAttributes

					oGTmp.Dispose();
					oBTmp.Dispose();
				}

				//oBaseImage.Save(context.Server.MapPath(resultImagePath), imgFormat);
				oResult = oBaseImage;

				oGraphics.Dispose();
				oOverImage.Dispose();
				//oBaseImage.Dispose();

				return true;
			}
			catch (Exception err)
			{
				oResult = null;//   ʱȭ
				return false;
			}
		}

		static public bool GenerateExtendedEdgeImage(string orgImagePath, 
			string resultImagePath, int edgeWidth, int edgeHeight,
			HttpContext context, ImageFormat imgFormat) {

			try {
				Bitmap oOrgImage = new Bitmap(context.Server.MapPath(orgImagePath));
				Bitmap oResultImage = new Bitmap(oOrgImage.Width + edgeWidth * 2, 
					oOrgImage.Height + edgeHeight * 2);

				// ̹  Graphics ü 
				Graphics oGraphics = Graphics.FromImage( oResultImage );

				Color orgColor = oOrgImage.GetPixel(1, 1);
				//Color orgColor = Color.FromArgb(255, 255, 255);

				SolidBrush solidBrush = new SolidBrush(orgColor);
				//oGraphics.FillEllipse(solidBrush, 0, 0, oResultImage.Width, oResultImage.Height);
				oGraphics.FillRectangle(solidBrush, 0, 0, oResultImage.Width, oResultImage.Height);

				/*
				//  .
				int r = (int)~orgColor.R;
				int g = (int)~orgColor.G;
				int b = (int)~orgColor.B;

				Color tranColor = Color.FromArgb(orgColor.A, r, g, b);
				*/

				Color tranColor = Color.FromArgb(0, 0, 0);
				Pen pen = new Pen(tranColor);

				oGraphics.DrawRectangle(pen, edgeWidth-1, edgeHeight-1, 
					oOrgImage.Width + 2, oOrgImage.Height + 2);

				oGraphics.DrawImage(oOrgImage, edgeWidth, edgeHeight);

				oResultImage.Save(context.Server.MapPath(resultImagePath), imgFormat);

				oGraphics.Dispose();
				oOrgImage.Dispose();
				oResultImage.Dispose();

				return true;
			}
			catch {
				return false;
			}
		}

		static public string GetImageNameFront(int no, string head, string tail) {
			string strNum = no.ToString().PadLeft(6, '0');
			strNum = strNum.Substring(strNum.Length-6);

			string ret = head + strNum + "_" + tail;

			return ret;
		}

		#endregion

		#region  ޼ҵ 
		private void Init(string baseDir, string imgHead, string altercastDirName,
			ImageFormat imgFmtL, ImageFormat imgFmtM, ImageFormat imgFmtS, 
			ArrayList kinds) 
		{
			_baseDir = baseDir[baseDir.Length-1] == '/' ? baseDir : baseDir+"/";
			_imgHead = imgHead;
			_altercastDirName = altercastDirName;
			_imgFormatL = imgFmtL;
			_imgFormatM = imgFmtM;
			_imgFormatS = imgFmtS;
			_kinds = kinds;

			_context = HttpContext.Current;

			if (!Directory.Exists(_context.Request.MapPath(AltercastDirPath))) {
				Directory.CreateDirectory(_context.Request.MapPath(AltercastDirPath));
			}

		}

		private ImageFormat GetImageFormat(string ext) 
		{
			if (ext == "gif")
				return ImageFormat.Gif;
			else if (ext == "jpg")
				return ImageFormat.Jpeg;
			else if (ext == "png")
				return ImageFormat.Png;
			else
				return null;
		}
		#endregion
	}

	public struct Kind {
		public string Nmae;
		public string Simpler;
	}

	public class GenerateParams {
		public uint AppliedImagePerSize;
		public byte StringUsingMethod;
		public string OverImageName;
		public string AppendString;
		public bool StringDependentOnImage;
		public int StringRotate;
		public int StringAlpha;
		public string StringColor;
		public string StringFont;
		public int StringSize;
		public int[] OverXY;
		public int OverRotate;
		public int OverAlpha;
		public int[] OverStringPosXY;
		public int OverPercent;
		public bool OverStringHAuto;
		public bool OverStringVAuto;
		public int OriginalSize;
	}
}
 