
/*
 * Contact Class.
 *   Version: 0.1.2
 *   Desc: Data type to store contact information. For simplicity,
 *         this class does not user getter/setter methods to access
 *         data. After initialization, properties are assigned directly by
 *         methods of the ContactManager class.
 */

 
 
/* =================
 * Class constructor
 * =================
 *   Desc: The Contact constructor is not called directly by the
 *         user, but by the addContact() method of the ContactManager class.
 *   Parameters:
 *     firstName    The person's first name.
 *     lastName     The person's last name.
 *     email        The person's email address.
 *     phone        The person's phone number.
 *     phonetype    The type of phone number (home, work, cell)
 *     company      The person's place of business.
 *     comments     Arbitrary notes associated with the person.
 *     id           A unique id by with to identify the contact record.
 * =================
 */
function Contact (firstName, 
                  lastName,
                  email, 
                  phone, 
                  phonetype, 
                  company, 
                  comments, 
                  id) {
  // Assign user-defined parameters to corresponding properties.
  this.firstName = (typeof firstName == "string") ? firstName : "";
  this.lastName = (typeof lastName == "string") ? lastName : "";
  this.email = (typeof email == "string") ? email : "";
  this.phone = (typeof phone == "string") ? phone : "";
  this.phonetype = (typeof phone == "string") ? phonetype : "";
  this.company = (typeof phone == "string") ? company : "";
  this.comments = (typeof phone == "string") ? comments : "";
  
  // Note the creation time for this contact.
  this.dateAdded = this.dateModified = new Date();
  
  // Assign the contact a unique id, provided by the addContact()
  // method of the ContactManager class.
  this.id = id;
}